<?php

declare(strict_types=1);

namespace Drupal\date_point\Plugin\views;

use DateTimeImmutable as DTI;
use Drupal\Core\Database\Database;
use Drupal\date_point\Data\Granularity;
use Drupal\date_point\Data\Precision;
use Drupal\date_point\Plugin\Field\FieldType\DateTime\DateTimeItem;

/**
 * A helper to build datetime in views plugins.
 *
 * @property array{dp_granularity: value-of<\Drupal\date_point\Data\Granularity>} $options
 */
trait DateTimeTrait {

  /**
   * {@selfdoc}
   */
  private function getGranularity(): Granularity {
    if (!isset($this->options['dp_granularity'])) {
      throw new \LogicException('Granularity option is not set.');
    }
    return Granularity::from($this->options['dp_granularity']);
  }

  /**
   * {@selfdoc}
   */
  public function buildValue(string $input): ?string {
    try {
      // Input time is supposed to be in default site timezone.
      $datetime = (new DTI($input))->setTimezone(new \DateTimeZone(DateTimeItem::STORAGE_TIMEZONE));
    }
    catch (\DateMalformedStringException) {
      return NULL;
    }
    $precision = match ($this->getGranularity()) {
      Granularity::MICROSECOND, Granularity::MILLISECOND => Precision::MICROSECONDS,
      default => Precision::SECONDS,
    };
    return $datetime->format(DateTimeItem::buildFormat($precision));
  }

  /**
   * {@selfdoc}
   */
  private function buildGranularityExpression(string $field, string $operator, string $value): string {
    switch (Database::getConnection()->databaseType()) {
      case 'mysql':
        $format = match ($this->getGranularity()) {
          Granularity::MICROSECOND,
          Granularity::MILLISECOND => '%Y-%m-%d %H:%i:%s.%f',
          Granularity::SECOND => '%Y-%m-%d %H:%i:%s',
          Granularity::MINUTE => '%Y-%m-%d %H:%i',
          Granularity::HOUR => '%Y-%m-%d %H',
          Granularity::DAY => '%Y-%m-%d',
          Granularity::MONTH => '%Y-%m',
          Granularity::YEAR => '%Y',
        };
        $snippet = "DATE_FORMAT($field, '$format') $operator DATE_FORMAT('$value', '$format')";
        break;

      case 'pgsql':
        // @todo Implement this.
        break;

      case 'sqlite':
        // @todo Implement this.
        break;

      default:
        throw new \LogicException('Unsupported database driver');
    }

    // @phpstan-ignore-next-line
    return $snippet;
  }

}
