<?php

declare(strict_types=1);

namespace Drupal\date_point\Plugin\views\filter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup as TM;
use Drupal\date_point\Data\Granularity;
use Drupal\date_point\Plugin\views\DateTimeTrait;
use Drupal\views\Plugin\views\filter\FilterPluginBase;

/**
 * Filter to handle dates points.
 *
 * @ViewsFilter("dp_date_time")
 *
 * @property \Drupal\views\Plugin\views\query\Sql $query
 * @property array{dp_granularity: value-of<\Drupal\date_point\Data\Granularity>, dp_widget: string, expose: array, exposed: bool, group: integer} $options
 */
final class DateTime extends FilterPluginBase {

  use DateTimeTrait;

  /**
   * {@inheritdoc}
   */
  protected $alwaysMultiple = TRUE;

  /**
   * {@inheritdoc}
   */
  protected function defineOptions(): array {
    $options = parent::defineOptions();
    $options['dp_widget'] = ['default' => 'datetime-local'];
    $options['dp_granularity'] = ['default' => Granularity::SECOND->value];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function operatorOptions(): array {
    $options = [
      '<' => new TM('Is earlier than'),
      '<=' => new TM('Is earlier than or equal to'),
      '=' => new TM('Is equal to'),
      '!=' => new TM('Is not equal to'),
      '>=' => new TM('Is later than or equal to'),
      '>' => new TM('Is later than'),
      'empty' => new TM('Is empty (NULL)'),
      'not_empty' => new TM('Is not empty (NOT NULL)'),
    ];
    if (!empty($this->definition['allow empty'])) {
      unset($options['empty'], $options['not_empty']);
    }
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state): void {
    parent::buildOptionsForm($form, $form_state);

    $form['dp_widget'] = [
      '#type' => 'select',
      '#title' => new TM('Widget'),
      '#options' => [
        'date' => new TM('Date'),
        'datetime-local' => new TM('Datetime'),
        'textfield' => new TM('Text field'),
      ],
      '#default_value' => $this->options['dp_widget'],
    ];

    $form['dp_granularity'] = [
      '#type' => 'select',
      '#title' => new TM('Granularity'),
      '#options' => Granularity::getOptions(),
      '#default_value' => $this->options['dp_granularity'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function valueForm(&$form, FormStateInterface $form_state): void {

    $form['value'] = [
      '#type' => 'date',
      '#title' => new TM('Value'),
      '#attributes' => ['type' => $this->options['dp_widget']],
      '#default_value' => $this->value,
    ];

    if ($this->options['dp_widget'] === 'datetime-local') {
      $form['value']['#step'] = match ($this->getGranularity()) {
        Granularity::MICROSECOND => 0.000_001,
        Granularity::MILLISECOND => 0.001,
        Granularity::SECOND => 1,
        Granularity::MINUTE => 60,
        Granularity::HOUR => 60 * 60,
        default => 24 * 60 * 60,
      };
    }

    if ($form_state->get('exposed')) {
      $identifier = $this->options['expose']['identifier'];
      $user_input = $form_state->getUserInput();
      if (!isset($user_input[$identifier])) {
        $user_input[$identifier] = $this->value;
        $form_state->setUserInput($user_input);
      }
    }

    $form['value']['#states']['invisible'][] = [':input[name="options[operator]"]' => ['value' => 'empty']];
    $form['value']['#states']['invisible'][] = [':input[name="options[operator]"]' => ['value' => 'not_empty']];
  }

  /**
   * {@inheritdoc}
   */
  public function adminSummary(): TM|string {
    if ($this->isAGroup()) {
      return new TM('grouped');
    }
    elseif ($this->options['exposed']) {
      return new TM('exposed');
    }
    return parent::adminSummary();
  }

  /**
   * {@inheritdoc}
   */
  public function validateExposeForm($form, FormStateInterface $form_state): void {
    parent::validate();
    $input = $form_state->getValue('options')['value'] ?? NULL;
    if ($input && !self::buildValue($input)) {
      $form_state->setError($form['value'], (string) new TM('Wrong date value.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function query(): void {
    $this->ensureMyTable();
    $field = "$this->tableAlias.$this->realField";

    switch ($this->operator) {
      case 'empty':
        $this->query->addWhere($this->options['group'], $field, NULL, 'IS NULL');
        break;

      case 'not_empty':
        $this->query->addWhere($this->options['group'], $field, NULL, 'IS NOT NULL');
        break;

      default:
        $value = $this->buildValue($this->value);
        if (!$value) {
          // As there no validation for user input we just filter out all
          // records when the supplied date is not valid.
          $this->query->addWhereExpression($this->options['group'], '1 = 2');
          return;
        }
        $this->query->addWhereExpression(
          $this->options['group'],
          $this->buildGranularityExpression($field, $this->operator, $value),
        );
    }
  }

}
